# workflow.py
# Workflow automation with PLAMS

import os, sys
import my_settings # my_settings.py contains some predefined settings


@add_to_class(AMSResults)
def get_dftb_excitations(results):
  if results.job.ok():
    exc_energies   = Units.convert(results.readrkf('Excitations SS A','excenergies','dftb'),'au','eV')
    oscillator_str = results.readrkf('Excitations SS A','oscillator strengths','dftb')
  else:
    exc_energies, oscillator_str = [], []
  return exc_energies, oscillator_str


@add_to_class(ADFResults)
def get_excitations(results):
  if results.job.ok():
    exc_energies   = Units.convert(results.readkf('Excitations SS A','excenergies'),'au','eV')
    oscillator_str = Units.convert(results.readkf('Excitations SS A','oscillator strengths'),'au','Debye')
  else:
    exc_energies, oscillator_str = [], []
  return exc_energies, oscillator_str


################################ Step 0 ################################

# Create a list of Molecules from the .xyz files in the directory 'molecules':

molecules_dir = 'molecules'
molecules = read_molecules(molecules_dir)


################################ Step 1 ################################

print("Step 1: optimize molecules and calculate excitations with DFTB")

promising_molecules = {}
for name, mol in molecules.items():
  # The settings are defined in the module ``my_settings``,
  # which we imported at the beginning of our script
  dftb_job = AMSJob(name='DFTB_'+name, molecule=mol, settings=my_settings.dftb_GO_plus_excitations())
  dftb_job.run()

  exc_energies, oscillator_str = dftb_job.results.get_dftb_excitations()

  # Check whether the excitation satisfies our condition:
  for e,o in zip(exc_energies, oscillator_str):
    if e < 6.0 and o > 1.0E-4:
      print ("Found a molecule with a promising excitation: " + name)
      promising_molecules[name] = dftb_job.results.get_main_molecule()
      break

print("Number of promising molecules: {}".format(len(promising_molecules)))


################################ Step 2 ################################

print("Step 2: Geometry optimizations with ADF")

optimized_molecules = {}
for name, mol in promising_molecules.items():
  adf_GO_job = ADFJob(name='GO_'+name, molecule=mol, settings=my_settings.adf_GO(basis='DZP', XC_fun=('LDA', '')))
  adf_GO_job.run()

  # Before reading the results, make sure that the job terminated normally
  if adf_GO_job.ok():
    optimized_molecules[name] = adf_GO_job.results.get_main_molecule()

# ################################ Step 3 ################################

print("Step 3: Calculate the excitations with ADF")

for name, mol in optimized_molecules.items():
  adf_exc_job = ADFJob(name='EXC_'+name, molecule=mol, settings=my_settings.adf_excitations(n_excitations=5))
  adf_exc_job.run()

  exc_energies, oscillator_str = adf_exc_job.results.get_excitations()

  good_excitations = []
  for e,o in zip(exc_energies, oscillator_str):
    if e < 4.0 and o > 1.0E-4:
      good_excitations.append((e,o))

  if good_excitations:
    print ("Found a molecule with a good excitation: " + name)
    print ("Excitations (energy [eV], oscillator strength):")
    for e,o in good_excitations:
      print("   {:10.6f}   {:10.8f}".format(e,o))
